/***************************************************************************
         kamevoiceconfig.cpp  -  Declaration of config widget cass
                             -------------------
    begin                : Mon Aug 11 14:10:15 CEST 2003
    copyright            : (C) 2003 by Thomas Thiel
    email                : kv@wolfchild.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kamevoiceconfig.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qtable.h>
#include <qheader.h>
#include <qlistview.h>
#include <qtextbrowser.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qpixmap.h>

#include "CGenericConfig.h"
#include "kamevoiceactioneditor.h"
#include "kamevoice_images.h"
#include "kamevoiceaboutdialog.h"

extern CGenericConfig *KameVoiceConf;

/* 
 *  Constructs a KameVoiceConfig as a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 */
KameVoiceConfig::KameVoiceConfig( GVListenerStatus *status, QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    gvls=status;

    if ( !name )
	setName( "KameVoiceConfig" );
    KameVoiceConfigLayout = new QGridLayout( this, 1, 1, 11, 6, "KameVoiceConfigLayout"); 

    layout7 = new QHBoxLayout( 0, 0, 6, "layout7");

    layout6 = new QVBoxLayout( 0, 0, 6, "layout6");

    layout5 = new QVBoxLayout( 0, 0, 6, "layout5"); 

    layout3 = new QVBoxLayout( 0, 0, 6, "layout3"); 

    led_group1 = new KLed( this, "led_group1" );
    led_group2 = new KLed( this, "led_group2" );
    led_group3 = new KLed( this, "led_group3" );
    led_group4 = new KLed( this, "led_group4" );
    led_all = new KLed( this, "led_all" );
    led_team = new KLed( this, "led_team" );
    led_mute = new KLed( this, "led_mute" );
    led_command = new KLed( this, "led_command" );

    led_group1->setLook(KLed::Sunken);
    led_group2->setLook(KLed::Sunken);
    led_group3->setLook(KLed::Sunken);
    led_group4->setLook(KLed::Sunken);
    led_all->setLook(KLed::Sunken);
    led_team->setLook(KLed::Sunken);
    led_mute->setLook(KLed::Sunken);
    led_command->setLook(KLed::Sunken);

    // set LED colors
    led_group1->setColor(Qt::yellow);
    led_group2->setColor(Qt::yellow);
    led_group3->setColor(Qt::yellow);
    led_group4->setColor(Qt::yellow);
    led_all->setColor(Qt::yellow);
    led_team->setColor(Qt::yellow);
    led_mute->setColor(Qt::red);
    led_command->setColor(Qt::green);


    // set initial LED status
    led_group1->off();
    led_group2->off();
    led_group3->off();
    led_group4->off();
    led_all->off();
    led_team->off();
    led_mute->off();
    led_command->off();



    QSpacerItem* spacer = new QSpacerItem( 0, 35, QSizePolicy::Minimum, QSizePolicy::Expanding );
    layout6->addItem(spacer);

    layout6->addWidget(led_group1);
    layout6->addWidget(led_group2);
    layout6->addWidget(led_group3);
    layout6->addWidget(led_group4);
    layout6->addWidget(led_all);
    layout6->addWidget(led_team);
    layout6->addWidget(led_command);
    layout6->addWidget(led_mute);

    tbl_assigner = new QTable( this, "tbl_assigner" );
    tbl_assigner->setNumCols( tbl_assigner->numCols() + 1 );
    tbl_assigner->horizontalHeader()->setLabel( tbl_assigner->numCols() - 1, tr( "Button" ) );
    tbl_assigner->setNumCols( tbl_assigner->numCols() + 1 );
    tbl_assigner->horizontalHeader()->setLabel( tbl_assigner->numCols() - 1, tr( "Action - On" ) );
    tbl_assigner->setNumCols( tbl_assigner->numCols() + 1 );
    tbl_assigner->horizontalHeader()->setLabel( tbl_assigner->numCols() - 1, tr( "Action - Off" ) );
    tbl_assigner->setNumRows( tbl_assigner->numRows() + 1 );
    tbl_assigner->verticalHeader()->setLabel( tbl_assigner->numRows() - 1, tr( "1" ) );
    tbl_assigner->setNumRows( tbl_assigner->numRows() + 1 );
    tbl_assigner->verticalHeader()->setLabel( tbl_assigner->numRows() - 1, tr( "2" ) );
    tbl_assigner->setNumRows( tbl_assigner->numRows() + 1 );
    tbl_assigner->verticalHeader()->setLabel( tbl_assigner->numRows() - 1, tr( "3" ) );
    tbl_assigner->setNumRows( tbl_assigner->numRows() + 1 );
    tbl_assigner->verticalHeader()->setLabel( tbl_assigner->numRows() - 1, tr( "4" ) );
    tbl_assigner->setNumRows( tbl_assigner->numRows() + 1 );
    tbl_assigner->verticalHeader()->setLabel( tbl_assigner->numRows() - 1, tr( "5" ) );
    tbl_assigner->setNumRows( tbl_assigner->numRows() + 1 );
    tbl_assigner->verticalHeader()->setLabel( tbl_assigner->numRows() - 1, tr( "6" ) );
    tbl_assigner->setNumRows( tbl_assigner->numRows() + 1 );
    tbl_assigner->verticalHeader()->setLabel( tbl_assigner->numRows() - 1, tr( "7" ) );
    tbl_assigner->setNumRows( tbl_assigner->numRows() + 1 );
    tbl_assigner->verticalHeader()->setLabel( tbl_assigner->numRows() - 1, tr( "8" ) );
    tbl_assigner->setNumRows( 8 );
    tbl_assigner->setNumCols( 3 );
    tbl_assigner->verticalHeader()->hide();
    tbl_assigner->setLeftMargin(0);

    tbl_assigner->setText(0,0,"Group 1");
    tbl_assigner->setText(1,0,"Group 2");
    tbl_assigner->setText(2,0,"Group 3");
    tbl_assigner->setText(3,0,"Group 4");
    tbl_assigner->setText(4,0,"All");
    tbl_assigner->setText(5,0,"Team");
    tbl_assigner->setText(6,0,"Command");
    tbl_assigner->setText(7,0,"Mute");

    tbl_assigner->setColumnReadOnly(0,true);

    layout7->addLayout(layout6);
    layout7->addWidget(tbl_assigner);

    layout3->addLayout( layout7 );

    layout2 = new QVBoxLayout( 0, 0, 6, "layout2"); 

    lv_actions = new QListView( this, "lv_actions" );
    lv_actions->addColumn( tr( "identifier" ) );
    lv_actions->addColumn( tr( "issued command" ) );
    layout2->addWidget( lv_actions );

    layout1 = new QHBoxLayout( 0, 0, 6, "layout1"); 

    btn_add = new QPushButton( this, "btn_add" );
    layout1->addWidget( btn_add );

    btn_delete = new QPushButton( this, "btn_delete" );
    layout1->addWidget( btn_delete );

    btn_edit = new QPushButton( this, "btn_edit" );
    layout1->addWidget( btn_edit );
    layout2->addLayout( layout1 );
    layout3->addLayout( layout2 );
    layout5->addLayout( layout3 );

    brws_docs = new QTextBrowser( this, "brws_docs" );
    layout5->addWidget( brws_docs );

    layout8 = new QHBoxLayout( 0, 0, 6, "layout8");

    btn_close = new QPushButton( this, "btn_close" );
    layout8->addWidget( btn_close );

    btn_quit = new QPushButton( this, "btn_quit" );
    layout8->addWidget( btn_quit );

    btn_about = new QPushButton( this, "btn_about" );
    layout8->addWidget( btn_about );

    layout5->addLayout( layout8 );

    KameVoiceConfigLayout->addLayout( layout5, 0, 0 );
    languageChange();
    resize( QSize(468, 620).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

    btn_close->setDefault(true);
    connect( btn_close, SIGNAL( clicked() ), this, SLOT( close_clicked() ) );
    connect( btn_add, SIGNAL( clicked() ), this, SLOT( add_clicked() ) );
    connect( btn_edit, SIGNAL( clicked() ), this, SLOT( edit_clicked() ) );
    connect( btn_delete, SIGNAL( clicked() ), this, SLOT( del_clicked() ) );
    connect( btn_quit, SIGNAL( clicked() ), this, SLOT( quit_clicked() ) );
    connect( btn_about, SIGNAL( clicked() ), this, SLOT( about_clicked() ) );

    list_ids = new QStringList();
            
    parseConfig();

    combo_group1 = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_group1->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group1"))
      combo_group1->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group1"));

    combo_group2 = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_group2->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group2"))
      combo_group2->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group2"));

    combo_group3 = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_group3->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group3"))
      combo_group3->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group3"));

    combo_group4 = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_group4->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group4"))
      combo_group4->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group4"));

    combo_all = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_all->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_all"))
      combo_all->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_all"));

    combo_team = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_team->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_team"))
      combo_team->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_team"));

    combo_command = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_command->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_command"))
      combo_command->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_command"));

    combo_mute = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_mute->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_mute"))
      combo_mute->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_mute"));

    combo_group1_off = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_group1_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group1_off"))
      combo_group1_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group1_off"));

    combo_group2_off = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_group2_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group2_off"))
      combo_group2_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group2_off"));

    combo_group3_off = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_group3_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group3_off"))
      combo_group3_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group3_off"));

    combo_group4_off = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_group4_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group4_off"))
      combo_group4_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group4_off"));

    combo_all_off = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_all_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_all_off"))
      combo_all_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_all_off"));

    combo_team_off = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_team_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_team_off"))
      combo_team_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_team_off"));

    combo_command_off = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_command_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_command_off"))
      combo_command_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_command_off"));

    combo_mute_off = new QComboTableItem(tbl_assigner,*list_ids,false);
    combo_mute_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_mute_off"))
      combo_mute_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_mute_off"));

    
    tbl_assigner->setItem(0,1,combo_group1);
    tbl_assigner->setItem(1,1,combo_group2);
    tbl_assigner->setItem(2,1,combo_group3);
    tbl_assigner->setItem(3,1,combo_group4);
    tbl_assigner->setItem(4,1,combo_all);
    tbl_assigner->setItem(5,1,combo_team);
    tbl_assigner->setItem(6,1,combo_command);
    tbl_assigner->setItem(7,1,combo_mute);

    tbl_assigner->setItem(0,2,combo_group1_off);
    tbl_assigner->setItem(1,2,combo_group2_off);
    tbl_assigner->setItem(2,2,combo_group3_off);
    tbl_assigner->setItem(3,2,combo_group4_off);
    tbl_assigner->setItem(4,2,combo_all_off);
    tbl_assigner->setItem(5,2,combo_team_off);
    tbl_assigner->setItem(6,2,combo_command_off);
    tbl_assigner->setItem(7,2,combo_mute_off);

    systray = new KSystemTray(this);
    systray->setPixmap(qembed_findImage("kamevoice"));
    systray->show();

    gvl= new GVListener(this,gvls);
    gvl->start();

    usleep(5000);

    if (!gvls->isRunning()) close();


}

/*
 *  Destroys the object and frees any allocated resources
 */
KameVoiceConfig::~KameVoiceConfig()
{
    delete list_ids;

    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void KameVoiceConfig::languageChange()
{
    setCaption( tr( "KameVoice - Configure" ) );
    tbl_assigner->horizontalHeader()->setLabel( 0, tr( "Button" ) );
    tbl_assigner->horizontalHeader()->setLabel( 1, tr( "Action - On" ) );
    tbl_assigner->horizontalHeader()->setLabel( 2, tr( "Action - Off" ) );
    tbl_assigner->verticalHeader()->setLabel( 0, tr( "1" ) );
    tbl_assigner->verticalHeader()->setLabel( 1, tr( "2" ) );
    tbl_assigner->verticalHeader()->setLabel( 2, tr( "3" ) );
    tbl_assigner->verticalHeader()->setLabel( 3, tr( "4" ) );
    tbl_assigner->verticalHeader()->setLabel( 4, tr( "5" ) );
    tbl_assigner->verticalHeader()->setLabel( 5, tr( "6" ) );
    tbl_assigner->verticalHeader()->setLabel( 6, tr( "7" ) );
    tbl_assigner->verticalHeader()->setLabel( 7, tr( "8" ) );
    lv_actions->header()->setLabel( 0, tr( "identifier" ) );
    lv_actions->header()->setLabel( 1, tr( "issued command" ) );
    btn_add->setText( tr( "Add" ) );
    btn_delete->setText( tr( "Delete" ) );
    btn_edit->setText( tr( "Edit" ) );
    brws_docs->setText( tr( "<H2>Controlling the issued commands in KameVoice</H2>\n"
                            "\n"
                            "<P>KameVoice controls its issued commands by Actions. Actions are nothing else but shell commands,\n"
                            "which are identified by a name, called the identifier.</P>\n"
                            "<P>Example:<br><br>identifier: xmms-play<br>command: xmms --play</P>\n"
                            "<P>You can control what ist happening simply by using the buttons under the action list. The use should be pretty\n"
                            "self-explanatory.</P>\n"
                            "\n") );
    btn_close->setText( tr( "Close" ) );
    btn_quit->setText( tr( "Quit" ) );
    btn_about->setText( tr( "About" ) );
}

/** Parses the configuration file for actions and inserts it into the 
widget for manipulation. */
void KameVoiceConfig::parseConfig()
{
  QString actionPrefix = "kamevoice_action";
  QString actionIdString = "_id_";
  QString actionCmdString = "_cmd_";

  int count_of_actions = 0;

  for (int i=0; i<=count_of_actions; i++)

  while ((KameVoiceConf->existValue(actionPrefix+actionIdString+QString::number(count_of_actions)))
        &(KameVoiceConf->existValue(actionPrefix+actionCmdString+QString::number(count_of_actions))))
  {
    QString momId="";
    QString momCmd="";

    momId = KameVoiceConf->getValue(actionPrefix+actionIdString+QString::number(count_of_actions));
    momCmd = KameVoiceConf->getValue(actionPrefix+actionCmdString+QString::number(count_of_actions));

    // yes, we can actually do it like this without dying a horrible death. Qt manages the reference
    // to this object internally, including the destruction process at the end. So please put away the torches
    // and the scythes !
    new QListViewItem(lv_actions,momId,momCmd);

    list_ids->append(momId);
    list_ids->sort();

                
    count_of_actions++;                      
  }


  list_ids->append("None");


}

/** Gets the new configuration data and stores it in the config file. */
void KameVoiceConfig::writeConfig()
{
  int cnt=0;

  QString actionPrefix = "kamevoice_action";
  QString actionIdString = "_id_";
  QString actionCmdString = "_cmd_";

  QListViewItem *momItem = NULL;
  
  // First, delete all old Entries in the action table.
  while ((KameVoiceConf->existValue(actionPrefix+actionIdString+QString::number(cnt)))
        |(KameVoiceConf->existValue(actionPrefix+actionCmdString+QString::number(cnt))))
  {
    KameVoiceConf->remValue(actionPrefix+actionIdString+QString::number(cnt));
    KameVoiceConf->remValue(actionPrefix+actionCmdString+QString::number(cnt));
    cnt++;
  }  

  momItem = lv_actions->firstChild();
  cnt=0;
  
  while (momItem)
  {
    KameVoiceConf->addValue(actionPrefix+actionIdString+QString::number(cnt),momItem->text(0));
    KameVoiceConf->addValue(actionPrefix+actionCmdString+QString::number(cnt),momItem->text(1));
    
    momItem=momItem->nextSibling();
    cnt++;
  }

  KameVoiceConf->addValue("kamevoice_action_group1",combo_group1->currentText());
  KameVoiceConf->addValue("kamevoice_action_group2",combo_group2->currentText());
  KameVoiceConf->addValue("kamevoice_action_group3",combo_group3->currentText());
  KameVoiceConf->addValue("kamevoice_action_group4",combo_group4->currentText());
  KameVoiceConf->addValue("kamevoice_action_all",combo_all->currentText());
  KameVoiceConf->addValue("kamevoice_action_team",combo_team->currentText());
  KameVoiceConf->addValue("kamevoice_action_mute",combo_mute->currentText());
  KameVoiceConf->addValue("kamevoice_action_command",combo_command->currentText());

  KameVoiceConf->addValue("kamevoice_action_group1_off",combo_group1_off->currentText());
  KameVoiceConf->addValue("kamevoice_action_group2_off",combo_group2_off->currentText());
  KameVoiceConf->addValue("kamevoice_action_group3_off",combo_group3_off->currentText());
  KameVoiceConf->addValue("kamevoice_action_group4_off",combo_group4_off->currentText());
  KameVoiceConf->addValue("kamevoice_action_all_off",combo_all_off->currentText());
  KameVoiceConf->addValue("kamevoice_action_team_off",combo_team_off->currentText());
  KameVoiceConf->addValue("kamevoice_action_mute_off",combo_mute_off->currentText());
  KameVoiceConf->addValue("kamevoice_action_command_off",combo_command_off->currentText());

  KameVoiceConf->writeConfig();
}
/** ends the dialog, but saves the settings before */
void KameVoiceConfig::close_clicked()
{
  writeConfig();
  hide();
}

/** Called if the add Action Button was clicked */
void KameVoiceConfig::add_clicked()
{
  QString momId;
  QString momCmd;

  KameVoiceActionEditor *kvae = new KameVoiceActionEditor(this);
  if ( kvae->exec() == QDialog::Accepted ) 
  {
    
    momId = kvae->getId();
    momCmd = kvae->getCmd();

    if (!isUniqueId(momId)) return;
    
    new QListViewItem(lv_actions,momId,momCmd);

    list_ids->append(momId);
    list_ids->sort();
            
    combo_all->setStringList(*list_ids);
    combo_team->setStringList(*list_ids);
    combo_mute->setStringList(*list_ids);
    combo_command->setStringList(*list_ids);
    combo_group1->setStringList(*list_ids);
    combo_group2->setStringList(*list_ids);
    combo_group3->setStringList(*list_ids);
    combo_group4->setStringList(*list_ids);

    combo_all_off->setStringList(*list_ids);
    combo_team_off->setStringList(*list_ids);
    combo_mute_off->setStringList(*list_ids);
    combo_command_off->setStringList(*list_ids);
    combo_group1_off->setStringList(*list_ids);
    combo_group2_off->setStringList(*list_ids);
    combo_group3_off->setStringList(*list_ids);
    combo_group4_off->setStringList(*list_ids);


    combo_group1->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group1"))
      combo_group1->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group1"));

    combo_group2->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group2"))
      combo_group2->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group2"));

    combo_group3->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group3"))
      combo_group3->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group3"));

    combo_group4->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group4"))
      combo_group4->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group4"));

    combo_all->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_all"))
      combo_all->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_all"));

    combo_team->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_team"))
      combo_team->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_team"));

    combo_command->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_command"))
      combo_command->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_command"));

    combo_mute->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_mute"))
      combo_mute->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_mute"));

    combo_group1_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group1_off"))
      combo_group1_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group1_off"));

    combo_group2_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group2_off"))
      combo_group2_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group2_off"));

    combo_group3_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group3_off"))
      combo_group3_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group3_off"));

    combo_group4_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_group4_off"))
      combo_group4_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group4_off"));

    combo_all_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_all_off"))
      combo_all_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_all_off"));

    combo_team_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_team_off"))
      combo_team_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_team_off"));

    combo_command_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_command_off"))
      combo_command_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_command_off"));

    combo_mute_off->setCurrentItem("None");
    if (KameVoiceConf->existValue("kamevoice_action_mute_off"))
      combo_mute_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_mute_off"));

  };
      
  delete kvae;
}
/** Called when the delete action button was clicked */
void KameVoiceConfig::del_clicked()
{
  QListViewItem *momItem;
  QStringList momStrList;
  QString delId;
  int cnt=0;
  
  momItem = lv_actions->selectedItem();
  if (!momItem) return;

  delId = momItem->text(0);

  momItem = lv_actions->firstChild();
  while (momItem)
  {
    if (momItem->text(0)!=delId)
    {
        momStrList.append(momItem->text(0));

        momItem = momItem->nextSibling();
        cnt++;
    }
    else
    {
        QListViewItem *delItem = momItem;
        
        momItem = momItem->nextSibling();

        delete delItem;

        cnt++;
        
    }
  }

  delete list_ids;

  list_ids = new QStringList(momStrList);
  list_ids->append("None");
  list_ids->sort();

  combo_all->setStringList(*list_ids);
  combo_team->setStringList(*list_ids);
  combo_mute->setStringList(*list_ids);
  combo_command->setStringList(*list_ids);
  combo_group1->setStringList(*list_ids);
  combo_group2->setStringList(*list_ids);
  combo_group3->setStringList(*list_ids);
  combo_group4->setStringList(*list_ids);

  combo_all_off->setStringList(*list_ids);
  combo_team_off->setStringList(*list_ids);
  combo_mute_off->setStringList(*list_ids);
  combo_command_off->setStringList(*list_ids);
  combo_group1_off->setStringList(*list_ids);
  combo_group2_off->setStringList(*list_ids);
  combo_group3_off->setStringList(*list_ids);
  combo_group4_off->setStringList(*list_ids);

  combo_group1->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_group1"))
    combo_group1->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group1"));

  combo_group2->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_group2"))
    combo_group2->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group2"));

  combo_group3->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_group3"))
    combo_group3->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group3"));

  combo_group4->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_group4"))
    combo_group4->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group4"));

  combo_all->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_all"))
    combo_all->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_all"));

  combo_team->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_team"))
    combo_team->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_team"));

  combo_command->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_command"))
    combo_command->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_command"));

  combo_mute->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_mute"))
    combo_mute->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_mute"));  

  combo_group1_off->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_group1_off"))
    combo_group1_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group1_off"));

  combo_group2_off->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_group2_off"))
    combo_group2_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group2_off"));

  combo_group3_off->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_group3_off"))
    combo_group3_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group3_off"));

  combo_group4_off->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_group4_off"))
    combo_group4_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group4_off"));

  combo_all_off->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_all_off"))
    combo_all_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_all_off"));

  combo_team_off->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_team_off"))
    combo_team_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_team_off"));

  combo_command_off->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_command_off"))
    combo_command_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_command_off"));

  combo_mute_off->setCurrentItem("None");
  if (KameVoiceConf->existValue("kamevoice_action_mute_off"))
    combo_mute_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_mute_off"));
      
}


/** called when the edit action button was clicked */
void KameVoiceConfig::edit_clicked()
{
  QListViewItem *momItem;
  QStringList momStrList;
  QString momId;
  QString momCmd;

  momItem = lv_actions->selectedItem();
  if (!momItem) return;

  momId = momItem->text(0);
  momCmd = momItem->text(1);

  KameVoiceActionEditor *kvae = new KameVoiceActionEditor(momId,momCmd,this);
  if ( kvae->exec() == QDialog::Accepted )
  {
    delete momItem;
    new QListViewItem(lv_actions,kvae->getId(),kvae->getCmd());

    momItem = lv_actions->firstChild();
    while (momItem)
    {
        momStrList.append(momItem->text(0));
        momItem = momItem->nextSibling();
    };
    
  };
  
  delete list_ids;

  list_ids = new QStringList(momStrList);
  list_ids->append("None");
  list_ids->sort();

  combo_all->setStringList(*list_ids);
  combo_team->setStringList(*list_ids);
  combo_mute->setStringList(*list_ids);
  combo_command->setStringList(*list_ids);
  combo_group1->setStringList(*list_ids);
  combo_group2->setStringList(*list_ids);
  combo_group3->setStringList(*list_ids);
  combo_group4->setStringList(*list_ids);

  combo_all_off->setStringList(*list_ids);
  combo_team_off->setStringList(*list_ids);
  combo_mute_off->setStringList(*list_ids);
  combo_command_off->setStringList(*list_ids);
  combo_group1_off->setStringList(*list_ids);
  combo_group2_off->setStringList(*list_ids);
  combo_group3_off->setStringList(*list_ids);
  combo_group4_off->setStringList(*list_ids);

  if (combo_group1->currentText()==momId)
  {
     combo_group1->setStringList(*list_ids);
     combo_group1->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_group1->setStringList(*list_ids);
     combo_group1->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_group1"))
       combo_group1->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group1"));

  };

  if (combo_group2->currentText()==momId)
  {
     combo_group2->setStringList(*list_ids);
     combo_group2->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_group2->setStringList(*list_ids);
     combo_group2->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_group2"))
       combo_group2->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group2"));

  };

  if (combo_group3->currentText()==momId)
  {
     combo_group3->setStringList(*list_ids);
     combo_group3->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_group3->setStringList(*list_ids);
     combo_group3->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_group3"))
       combo_group3->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group3"));

  };

  if (combo_group4->currentText()==momId)
  {
     combo_group4->setStringList(*list_ids);
     combo_group4->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_group4->setStringList(*list_ids);
     combo_group4->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_group4"))
       combo_group4->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group4"));

  };

  if (combo_all->currentText()==momId)
  {
     combo_all->setStringList(*list_ids);
     combo_all->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_all->setStringList(*list_ids);
     combo_all->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_all"))
       combo_all->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_all"));

  };

  if (combo_team->currentText()==momId)
  {
     combo_team->setStringList(*list_ids);
     combo_team->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_team->setStringList(*list_ids);
     combo_team->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_team"))
       combo_team->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_team"));

  };

  if (combo_mute->currentText()==momId)
  {
     combo_mute->setStringList(*list_ids);
     combo_mute->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_mute->setStringList(*list_ids);
     combo_mute->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_mute"))
       combo_mute->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_mute"));

  };

  if (combo_command->currentText()==momId)
  {
     combo_command->setStringList(*list_ids);
     combo_command->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_command->setStringList(*list_ids);
     combo_command->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_command"))
       combo_command->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_command"));

  };

  if (combo_group1_off->currentText()==momId)
  {
     combo_group1_off->setStringList(*list_ids);
     combo_group1_off->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_group1_off->setStringList(*list_ids);
     combo_group1_off->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_group1_off"))
       combo_group1_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group1_off"));

  };

  if (combo_group2_off->currentText()==momId)
  {
     combo_group2_off->setStringList(*list_ids);
     combo_group2_off->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_group2_off->setStringList(*list_ids);
     combo_group2_off->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_group2_off"))
       combo_group2_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group2_off"));

  };

  if (combo_group3_off->currentText()==momId)
  {
     combo_group3_off->setStringList(*list_ids);
     combo_group3_off->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_group3_off->setStringList(*list_ids);
     combo_group3_off->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_group3_off"))
       combo_group3_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group3_off"));

  };

  if (combo_group4_off->currentText()==momId)
  {
     combo_group4_off->setStringList(*list_ids);
     combo_group4_off->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_group4_off->setStringList(*list_ids);
     combo_group4_off->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_group4_off"))
       combo_group4_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_group4_off"));

  };

  if (combo_all_off->currentText()==momId)
  {
     combo_all_off->setStringList(*list_ids);
     combo_all_off->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_all_off->setStringList(*list_ids);
     combo_all_off->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_all_off"))
       combo_all_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_all_off"));

  };

  if (combo_team_off->currentText()==momId)
  {
     combo_team_off->setStringList(*list_ids);
     combo_team_off->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_team_off->setStringList(*list_ids);
     combo_team_off->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_team_off"))
       combo_team_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_team_off"));

  };

  if (combo_mute_off->currentText()==momId)
  {
     combo_mute_off->setStringList(*list_ids);
     combo_mute_off->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_mute_off->setStringList(*list_ids);
     combo_mute_off->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_mute_off"))
       combo_mute_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_mute_off"));

  };

  if (combo_command_off->currentText()==momId)
  {
     combo_command_off->setStringList(*list_ids);
     combo_command_off->setCurrentItem(kvae->getId());
  }
  else
  {
     combo_command_off->setStringList(*list_ids);
     combo_command_off->setCurrentItem("None");
     if (KameVoiceConf->existValue("kamevoice_action_command_off"))
       combo_command_off->setCurrentItem(KameVoiceConf->getValue("kamevoice_action_command_off"));

  };
    
  
}
/** checks if a given String is a valid unique identifier for an action. */
bool KameVoiceConfig::isUniqueId(QString id)
{
  QStringList momStrList;
  momStrList = list_ids->grep(id);
  if (momStrList.count()==0)
    return true;
  else
    return false;
}

/** called when quit button is clicked */
void KameVoiceConfig::quit_clicked()
{
    if (QMessageBox::warning(
            this,
            tr("KameVoice - Really Quit ?"),
            tr("Do you really want to quit?\n"
               "To hide this window, use the Close Button\n"
               "Your GameVoice buttons won't work without a running KameVoice instance!\n"
               "If minimized per close, you can alway access this dialog through the gamevoice systemtray icon.\n"),
            tr("&Nooooo, I'm a jerk !"), tr("OK, just &Quit !"),
            QString::null, 0, 1 ) )
    {
/*      writeConfig();
      gvls->terminateListener();
      while (gvls->isRunning()) sleep(1);*/
      close();
    };
}

/** called when about button is clicked */
void KameVoiceConfig::about_clicked()
{
  KameVoiceAboutDialog *kvad = new KameVoiceAboutDialog(this);
  kvad->exec();
  delete kvad;
}

void KameVoiceConfig::customEvent(QCustomEvent *e){

  // Look if we can identify the Event as GVEvent
  if (e->type()==42042)
  {
     GVEvent *gv_ev = (GVEvent*)e;

     if (gv_ev->getKey() == GVEvent::all) toggle_all(gv_ev->getStatus());
     if (gv_ev->getKey() == GVEvent::command) toggle_command(gv_ev->getStatus());
     if (gv_ev->getKey() == GVEvent::team) toggle_team(gv_ev->getStatus());
     if (gv_ev->getKey() == GVEvent::mute) toggle_mute(gv_ev->getStatus());
     if (gv_ev->getKey() == GVEvent::group1) toggle_group1(gv_ev->getStatus());
     if (gv_ev->getKey() == GVEvent::group2) toggle_group2(gv_ev->getStatus());
     if (gv_ev->getKey() == GVEvent::group3) toggle_group3(gv_ev->getStatus());
     if (gv_ev->getKey() == GVEvent::group4) toggle_group4(gv_ev->getStatus());
  };

}

/** No descriptions */
void KameVoiceConfig::toggle_group1(bool status)
{
    if (status)
    {
      QString cmd="";
      QListViewItem *momItem;

      led_group1->on();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_group1->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    }
    else
    {
      QString cmd="";
      QListViewItem *momItem;

      led_group1->off();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_group1_off->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    };
}

/** No descriptions */
void KameVoiceConfig::toggle_group2(bool status)
{
    if (status)
    {
      QString cmd="";
      QListViewItem *momItem;

      led_group2->on();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_group2->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    }
    else
    {
      QString cmd="";
      QListViewItem *momItem;

      led_group2->off();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_group2_off->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    };
}

/** No descriptions */
void KameVoiceConfig::toggle_group3(bool status)
{
    if (status)
    {
      QString cmd="";
      QListViewItem *momItem;

      led_group3->on();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_group3->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    }
    else
    {
      QString cmd="";
      QListViewItem *momItem;

      led_group3->off();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_group3_off->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    };
}

/** No descriptions */
void KameVoiceConfig::toggle_group4(bool status)
{
    if (status)
    {
      QString cmd="";
      QListViewItem *momItem;

      led_group4->on();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_group4->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    }
    else
    {
      QString cmd="";
      QListViewItem *momItem;

      led_group4->off();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_group4_off->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    };
}

/** No descriptions */
void KameVoiceConfig::toggle_all(bool status)
{
    if (status)
    {
      QString cmd="";
      QListViewItem *momItem;

      led_all->on();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_all->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    }
    else
    {
      QString cmd="";
      QListViewItem *momItem;

      led_all->off();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_all_off->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    };
}

/** No descriptions */
void KameVoiceConfig::toggle_team(bool status)
{
    if (status)
    {
      QString cmd="";
      QListViewItem *momItem;

      led_team->on();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_team->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    }
    else
    {
      QString cmd="";
      QListViewItem *momItem;

      led_team->off();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_team_off->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    };
}

/** No descriptions */
void KameVoiceConfig::toggle_mute(bool status)
{
    if (status)
    {
      QString cmd="";
      QListViewItem *momItem;

      led_mute->on();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_mute->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    }
    else
    {
      QString cmd="";
      QListViewItem *momItem;

      led_mute->off();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_mute_off->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    };
}

/** No descriptions */
void KameVoiceConfig::toggle_command(bool status)
{
    if (status)
    {
      QString cmd="";
      QListViewItem *momItem;

      led_command->on();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_command->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    }
    else
    {
      QString cmd="";
      QListViewItem *momItem;

      led_command->off();

      momItem = lv_actions->firstChild();
      if (!momItem) return;

      while (momItem)
      {
        if (momItem->text(0)==combo_command_off->currentText())
          cmd = momItem->text(1);

        momItem = momItem->nextSibling();
      };

      if ((cmd!="")|(cmd!="None")) system(cmd);
    };
}
/** called if the close widget event is recvd.  */
void KameVoiceConfig::closeEvent(QCloseEvent *e)
{
      writeConfig();
      gvls->terminateListener();
      while (gvls->isRunning()) sleep(1);
      e->accept();
}
