/***************************************************************************
           kamevoiceconfig.h  -  Definition of config widget cass
                             -------------------
    begin                : Mon Aug 11 14:10:15 CEST 2003
    copyright            : (C) 2003 by Thomas Thiel
    email                : kv@wolfchild.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KAMEVOICECONFIG_H
#define KAMEVOICECONFIG_H

#include <qvariant.h>
#include <qwidget.h>
#include <kled.h>
#include <ksystemtray.h>
#include "gvlistener.h"
#include "gvlistenerstatus.h"

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QTable;
class QComboTableItem;
class QListView;
class QListViewItem;
class QPushButton;
class QTextBrowser;

class KameVoiceConfig : public QWidget
{
    Q_OBJECT

public:
    KameVoiceConfig( GVListenerStatus *status, QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~KameVoiceConfig();
  /** Gets the new configuration data and stores it in the config file. */
  void writeConfig();

    QTable* tbl_assigner;
    QListView* lv_actions;
    QPushButton* btn_add;
    QPushButton* btn_delete;
    QPushButton* btn_edit;
    QTextBrowser* brws_docs;
    QPushButton* btn_close;
    QPushButton* btn_quit;
    QPushButton* btn_about;

    KLed* led_group1;
    KLed* led_group2;
    KLed* led_group3;
    KLed* led_group4;
    KLed* led_team;
    KLed* led_all;
    KLed* led_mute;
    KLed* led_command;

protected:
    QGridLayout* KameVoiceConfigLayout;
    QHBoxLayout* layout8;
    QHBoxLayout* layout7;
    QVBoxLayout* layout6;
    QVBoxLayout* layout5;
    QVBoxLayout* layout3;
    QVBoxLayout* layout2;
    QHBoxLayout* layout1;
    KSystemTray *systray;

    GVListener *gvl;

protected slots:
    virtual void languageChange();
    virtual void customEvent(QCustomEvent *e);

private: // Private methods
  /** Parses the configuration file for actions and inserts it into the 
widget for manipulation. */
  virtual void parseConfig();
  /** checks if a given String is a valid unique identifier for an action. */
  bool isUniqueId(QString id);

    QStringList *list_ids;
  
    QComboTableItem *combo_group1;
    QComboTableItem *combo_group2;
    QComboTableItem *combo_group3;
    QComboTableItem *combo_group4;
    QComboTableItem *combo_all;
    QComboTableItem *combo_team;
    QComboTableItem *combo_mute;
    QComboTableItem *combo_command;

    QComboTableItem *combo_group1_off;
    QComboTableItem *combo_group2_off;
    QComboTableItem *combo_group3_off;
    QComboTableItem *combo_group4_off;
    QComboTableItem *combo_all_off;
    QComboTableItem *combo_team_off;
    QComboTableItem *combo_mute_off;
    QComboTableItem *combo_command_off;

    GVListenerStatus *gvls;
public slots: // Public slots
  /** ends the dialog, but saves the settings before */
  void close_clicked();
  /** called when about button is clicked */
  void about_clicked();
  /** Called when Quit Button is clicked */
  void quit_clicked();
  /** No descriptions */
  virtual void toggle_group1(bool status);
  virtual void toggle_group2(bool status);
  virtual void toggle_group3(bool status);
  virtual void toggle_group4(bool status);
  virtual void toggle_all(bool status);
  virtual void toggle_team(bool status);
  virtual void toggle_mute(bool status);
  virtual void toggle_command(bool status);
  /** called when the edit action button was clicked */
  virtual void edit_clicked();
  /** Called when the delete action button was clicked */
  virtual void del_clicked();
  /** Called if the add Action Button was clicked */
  virtual void add_clicked();
  /** called if the close widget event is recvd.  */
  void closeEvent(QCloseEvent *e);
};

#endif // KAMEVOICECONFIG_H
